package com.jz.dmp.modules.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jz.agent.service.DmpDsAgentService;
import com.jz.common.constant.JsonResult;
import com.jz.common.constant.ResultCode;
import com.jz.common.page.PageInfoResponse;
import com.jz.common.utils.JsonMapper;
import com.jz.dmp.agent.DmpAgentResult;
import com.jz.dmp.modules.controller.DataIntegration.bean.DataSourceListDto;
import com.jz.dmp.modules.controller.DataIntegration.bean.DataSourceListReq;
import com.jz.dmp.modules.controller.DataIntegration.bean.DmpSyncingDatasourceReq;
import com.jz.dmp.modules.controller.DataIntegration.bean.SyncingDatasourceTypeDto;
import com.jz.dmp.modules.dao.DmpProjectDao;
import com.jz.dmp.modules.dao.DmpSyncingDatasourceDao;
import com.jz.dmp.modules.model.DmpAgentDatasourceInfo;
import com.jz.dmp.modules.model.DmpProjectSystemInfo;
import com.jz.dmp.modules.model.DmpSyncingDatasource;
import com.jz.dmp.modules.model.DmpSyncingDatasourceType;
import com.jz.dmp.modules.service.DmpSyncingDatasourceService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

/**
 * 数据源配置(DmpSyncingDatasource)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-24 14:09:59
 */
@Service("dmpSyncingDatasourceService")
@Transactional
public class DmpSyncingDatasourceServiceImpl implements DmpSyncingDatasourceService {

    private static Logger logger = LoggerFactory.getLogger(DmpSyncingDatasourceServiceImpl.class);

    @Autowired
    private DmpSyncingDatasourceDao dmpSyncingDatasourceDao;

    @Autowired
    private DmpProjectDao dmpProjectDao;

    @Autowired
    private DmpDsAgentService dmpDsAgentServiceImp;

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @Override
    public DmpSyncingDatasource queryById(Integer id) {
        return this.dmpSyncingDatasourceDao.queryById(id);
    }

    /**
     * 查询多条数据
     *
     * @param offset 查询起始位置
     * @param limit  查询条数
     * @return 对象列表
     */
    @Override
    public List<DmpSyncingDatasource> queryAllByLimit(int offset, int limit) {
        return this.dmpSyncingDatasourceDao.queryAllByLimit(offset, limit);
    }

    /**
     * 新增数据
     *
     * @param dmpSyncingDatasource 实例对象
     * @return 实例对象
     */
    @Override
    public DmpSyncingDatasource insert(DmpSyncingDatasource dmpSyncingDatasource) {
        this.dmpSyncingDatasourceDao.insert(dmpSyncingDatasource);
        return dmpSyncingDatasource;
    }

    /**
     * 修改数据
     *
     * @param dmpSyncingDatasource 实例对象
     * @return 实例对象
     */
    @Override
    public DmpSyncingDatasource update(DmpSyncingDatasource dmpSyncingDatasource) {
        this.dmpSyncingDatasourceDao.update(dmpSyncingDatasource);
        return this.queryById(dmpSyncingDatasource.getId());
    }

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 是否成功
     */
    @Override
    public boolean deleteById(Integer id) {
        return this.dmpSyncingDatasourceDao.deleteById(id) > 0;
    }

    /**
     * 数据源列表查询
     *
     * @param req
     */
    @Override
    public PageInfoResponse<DataSourceListDto> queryDataSourceListPage(DataSourceListReq req, HttpServletRequest httpRequest) throws Exception {
        PageInfoResponse<DataSourceListDto> pageInfoResponse = new PageInfoResponse<>();

        PageHelper.startPage(req.getPageNum(), req.getPageSize());
        req.setDatasourceName(req.getDatasourceName().trim());
        List<DataSourceListDto> list = dmpSyncingDatasourceDao.queryDataSourceListPage(req);
        PageInfo<DataSourceListDto> pageInfo = new PageInfo<>(list);

        pageInfoResponse.setCode(ResultCode.SUCCESS);
        pageInfoResponse.setMessage("查询成功");
        pageInfoResponse.setData(pageInfo);
        return pageInfoResponse;
    }

    /**
     * 批量删除数据源
     *
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public JsonResult delDataSourceById(String datasourceId) throws Exception {
        String[] ids = datasourceId.split(",");
        dmpSyncingDatasourceDao.delDataSourceById(ids);
        return new JsonResult();
    }

    @Override
    public JsonResult queryDatasourceType() throws Exception {
        List<Map> list = dmpSyncingDatasourceDao.queryDatasourceType();
        return new JsonResult(list);
    }

    /**
     * 新增-获取数据源类型
     *
     * @return
     */
    @Override
    public JsonResult queryGroupDatasourceType() throws Exception {
        Map map = new HashMap();
        Map<String, ArrayList<Map>> returnMap = new HashMap();
        List<Map> list = dmpSyncingDatasourceDao.queryGroupDatasourceType(map);
        if (list.size() > 0 && list != null) {
            for (Map<String, Object> dto : list) {
                String datasourceCatename = (String) dto.get("datasourceCatename");
                if (returnMap.get(datasourceCatename) == null) {
                    returnMap.put(datasourceCatename, new ArrayList<Map>());
                }
                ArrayList<Map> param = returnMap.get(datasourceCatename);
                if (dto.get("dbAttrs") != null) {
                    JSONObject dbAttrs = JSON.parseObject(dto.get("dbAttrs").toString());
                    dto.put("dbAttrs", dbAttrs);
                }
                param.add(dto);
            }
        }
        return new JsonResult(returnMap);
    }

    /**
     * 保存数据源
     *
     * @return
     */
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    @Override
    public JsonResult addDatasourceInfo(DmpSyncingDatasourceReq saveBody) throws Exception {
        if (StringUtils.isEmpty(saveBody.getDatasourceName())) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "数据源名称不能为空");
        }
        //通过名称查询数据源是否存在
        int len = dmpSyncingDatasourceDao.countDatasourceByName(saveBody.getDatasourceName(), saveBody.getProjectId());
        if (len > 0) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "数据源名称已存在");
        }
        DmpSyncingDatasource dsd = new DmpSyncingDatasource();
        BeanUtils.copyProperties(saveBody, dsd);

        dsd.setCreateTime(new Date());
        dsd.setDatasourceType(Integer.valueOf(saveBody.getDatasourceType())); //数据源类型ID
        dsd.setProjectId(Integer.valueOf(saveBody.getProjectId()));
        dsd.setDataStatus("1");
        dmpSyncingDatasourceDao.insert(dsd);
        return new JsonResult(dsd);
    }

    /**
     * 测试连通性
     *
     * @return
     */
    @Override
    public JsonResult testConnection(DmpSyncingDatasourceReq saveBody) throws Exception {
        DmpAgentDatasourceInfo ds = this.dsInfoDTO(saveBody); //查询数据源 对应的 数据库信息
        DmpAgentResult rst = dmpDsAgentServiceImp.testConnect(ds); //连接测试
        if (!rst.getCode().val().equals("200")) {
            return new JsonResult(rst.getCode(), rst.getMessage());
        } else {
            rst.setResult(JsonMapper.fromJsonString(rst.getMessage(), Boolean.class));
            return new JsonResult(ResultCode.SUCCESS, rst);
        }
    }

    @Override
    public JsonResult selectDataSourceInfoById(Map map) throws Exception {
        DataSourceListDto asd = dmpSyncingDatasourceDao.selectDataSourceInfoById(map);
        return new JsonResult(asd);
    }

    /**
     * 编辑数据源
     *
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public JsonResult updateDatasourceById(DmpSyncingDatasourceReq saveBody) throws Exception {
        DmpSyncingDatasource dsd = new DmpSyncingDatasource();
        BeanUtils.copyProperties(saveBody, dsd);

        dsd.setUpdateTime(new Date());
        if (StringUtils.isNotEmpty(saveBody.getDatasourceType())) {
            dsd.setDatasourceType(Integer.valueOf(saveBody.getDatasourceType())); //数据源类型ID
        }
        if (StringUtils.isNotEmpty(saveBody.getProjectId())) {
            dsd.setProjectId(Integer.valueOf(saveBody.getProjectId()));
        }
        if (StringUtils.isNotEmpty(saveBody.getDatasourceId())) { //数据源id
            dsd.setId(Integer.valueOf(saveBody.getDatasourceId()));
        }
        dsd.setDataStatus("1");

        int len = dmpSyncingDatasourceDao.update(dsd);
        if (len > 0) {
            return new JsonResult();
        } else {
            return new JsonResult(ResultCode.INTERNAL_SERVER_ERROR);
        }
    }

    @Override
    public List<DmpSyncingDatasource> findListByParams(DmpSyncingDatasource ds) throws Exception {
        return dmpSyncingDatasourceDao.findListByParams(ds);
    }

    /**
     * 获取数据源类型输入框属性
     *
     * @return
     */
    @Override
    public JsonResult<SyncingDatasourceTypeDto> selectDatasourceTypeAttrById(String datasourceTypeId) throws Exception {
        Map map = new HashMap();
        map.put("datasourceTypeId", datasourceTypeId);
        SyncingDatasourceTypeDto dto = new SyncingDatasourceTypeDto();
        List<Map> list = dmpSyncingDatasourceDao.queryGroupDatasourceType(map);
        if (CollectionUtils.isNotEmpty(list)) {
            Map str = list.get(0);
            if (StringUtils.isNotEmpty(str.get("datasourceTypeId").toString())) {
                dto.setDatasourceTypeId(str.get("datasourceTypeId").toString());
            }
            if (StringUtils.isNotEmpty(str.get("datasourceCatecode").toString())) {
                dto.setDatasourceCatecode(str.get("datasourceCatecode").toString());
            }
            if (StringUtils.isNotEmpty(str.get("datasourceCatename").toString())) {
                dto.setDatasourceCatename(str.get("datasourceCatename").toString());
            }
            if (StringUtils.isNotEmpty(str.get("datasourceTypeName").toString())) {
                dto.setDatasourceTypeName(str.get("datasourceTypeName").toString());
            }
            if (StringUtils.isNotEmpty(str.get("imgUrl").toString())) {
                dto.setImgUrl(str.get("imgUrl").toString());
            }
            if (StringUtils.isNotEmpty(str.get("dbAttrs").toString())) {
                dto.setDbAttrs(JSONObject.parseObject(str.get("dbAttrs").toString()));
            }
        }
        return new JsonResult(dto);
    }

    /*
     * 查询数据源 对应的 数据库信息
     * */
    private DmpAgentDatasourceInfo dsInfoDTO(DmpSyncingDatasourceReq body) throws Exception {
        //数据源类型ID去查询
        DmpSyncingDatasourceType type = dmpSyncingDatasourceDao.queryDatasourceTypeById(body.getDatasourceType());
        DmpAgentDatasourceInfo ds = new DmpAgentDatasourceInfo();
        ds.setDatasourceType(type.getDatasourceType());
        ds.setDatasourceName(body.getDatasourceName());
        ds.setDbName(body.getDbName());
        ds.setJdbcUrl(body.getJdbcUrl());
        ds.setUserName(body.getUserName());
        ds.setPassword(body.getPassword());
        ds.setDriverClassName(type.getDriverClassName());
        ds.setHost(body.getHost());
        ds.setPort(body.getPort());
        ds.setDelimiter(body.getDelimiter());
        ds.setProtocol(body.getProtocol());
        ds.setProjectId(body.getProjectId() + "");
        ds.setAccessId(body.getAccessId());
        ds.setAccessKey(body.getAccessKey());
        ds.setIsHaveHeader(body.getIsHaveHeader());
        ds.setEndpoint(body.getEndpoint());
        ds.setDefaultFs(body.getDefaultFs());

        DmpProjectSystemInfo info = dmpProjectDao.queryProjectSystemInfo(Long.valueOf(body.getProjectId()));
        ds.setKerberosIsenable(info.getKerberosIsenable());
        ds.setKerberosJaasConf(info.getKerberosJaasConf());
        ds.setKerberosKrb5Conf(info.getKerberosKrb5Conf());
        ds.setKerberosFqdn(info.getKerberosFqdn());
        ds.setHdfsDefaultConf(""); // info.getHdfsDefaultConf());
        ds.setHdfsConfPath(""); // info.getHdfsConfPath());
        ds.setHdfsAuthPath(body.getAccessKey());
        ds.setHdfsUserName(info.getHdfsUserName());
        ds.setHdfsSyncingPath(info.getHdfsSyncingPath());
        return ds;
    }
}